@@ -18,7 +18,6 @@ from idds.agents.common.baseagent import BaseAgent
 from idds.agents.common.eventbus.event import (EventType,
                                                NewProcessingEvent,
                                                PreparedProcessingEvent,
-                                               SyncProcessingEvent,
                                                UpdateTransformEvent)
 
 from .utils import handle_new_processing, handle_prepared_processing
@@ -34,13 +33,14 @@ class Submitter(Poller):
     """
 
     def __init__(self, num_threads=1, max_number_workers=3, poll_period=10, retries=3, retrieve_bulk_size=2,
-                 name='Submitter', message_bulk_size=1000, **kwargs):
+                 name='Submitter', use_process_pool=False, message_bulk_size=1000, **kwargs):
         self.max_number_workers = max_number_workers
         self.set_max_workers()
         num_threads = self.max_number_workers
 
         super(Submitter, self).__init__(num_threads=num_threads, max_number_workers=self.max_number_workers,
-                                        name=name, retrieve_bulk_size=retrieve_bulk_size, **kwargs)
+                                        name=name, use_process_pool=use_process_pool,
+                                        retrieve_bulk_size=retrieve_bulk_size, **kwargs)
         self.site_to_cloud = None
 
         self._new_processing_status = None
@@ -62,21 +62,17 @@ class Submitter(Poller):
             processing_status = [ProcessingStatus.New]
             self._new_processing_status = processing_status
             processings = core_processings.get_processings_by_status(status=processing_status, locking=True,
-                                                                     not_lock=True,
-                                                                     new_poll=True, only_return_id=True,
+                                                                     new_poll=True,
                                                                      min_request_id=BaseAgent.min_request_id,
-                                                                     bulk_size=self.retrieve_bulk_size)
+                                                                     bulk_size=self.get_bulk_size())
 
             # self.logger.debug("Main thread get %s [new] processings to process" % len(processings))
             if processings:
-                self.logger.info("Main thread get [new] processings to process: %s" % str(processings))
+                processing_ids = [pr['processing_id'] for pr in processings]
+                self.logger.info("Main thread get [new] processings to process: %s" % str(processing_ids))
 
-            events = []
-            for pr_id in processings:
-                self.logger.info("NewProcessingEvent(processing_id: %s)" % pr_id)
-                event = NewProcessingEvent(publisher_id=self.id, processing_id=pr_id)
-                events.append(event)
-            self.event_bus.send_bulk(events)
+            for pr in processings:
+                self.submit(self.process_new_processing, **{"processing": pr})
 
             return processings
         except exceptions.DatabaseException as ex:
@@ -104,21 +100,17 @@ class Submitter(Poller):
             processing_status = [ProcessingStatus.Prepared]
             self._prepared_processing_status = processing_status
             processings = core_processings.get_processings_by_status(status=processing_status, locking=True,
-                                                                     not_lock=True,
-                                                                     new_poll=True, only_return_id=True,
+                                                                     new_poll=True,
                                                                      min_request_id=BaseAgent.min_request_id,
-                                                                     bulk_size=self.retrieve_bulk_size)
+                                                                     bulk_size=self.get_bulk_size())
 
             # self.logger.debug("Main thread get %s [new] processings to process" % len(processings))
             if processings:
-                self.logger.info("Main thread get [prepared] processings to process: %s" % str(processings))
+                processing_ids = [pr['processing_id'] for pr in processings]
+                self.logger.info("Main thread get [prepared] processings to process: %s" % str(processing_ids))
 
-            events = []
-            for pr_id in processings:
-                self.logger.info("PreparedProcessingEvent(processing_id: %s)" % pr_id)
-                event = PreparedProcessingEvent(publisher_id=self.id, processing_id=pr_id)
-                events.append(event)
-            self.event_bus.send_bulk(events)
+            for pr in processings:
+                self.submit(self.process_prepared_processing, **{"processing": pr})
 
             return processings
         except exceptions.DatabaseException as ex:
@@ -157,7 +149,7 @@ class Submitter(Poller):
             self.logger.error(ex)
         return None
 
-    def handle_new_processing(self, processing):
+    def handle_new_processing(self, processing, check_previous=True):
         try:
             log_prefix = self.get_log_prefix(processing)
 
@@ -168,33 +160,85 @@ class Submitter(Poller):
             if self.enable_executors:
                 executors = self.get_extra_executors()
 
-            ret_new_processing = handle_new_processing(processing,
-                                                       self.agent_attributes,
-                                                       func_site_to_cloud=self.get_site_to_cloud,
-                                                       max_updates_per_round=self.max_updates_per_round,
-                                                       executors=executors,
-                                                       logger=self.logger,
-                                                       log_prefix=log_prefix)
-            status, processing, update_colls, new_contents, new_input_dependency_contents, msgs, errors = ret_new_processing
+            if check_previous:
+                pre_works_are_ok = True
+                if processing['parent_internal_id']:
+                    parent_internal_ids = processing['parent_internal_id'].split(",")
+                    prs = core_processings.get_processings(
+                        request_id=processing['request_id'],
+                        internal_ids=parent_internal_ids,
+                        loop_index=processing['loop_index']
+                    )
+                    if not prs:
+                        pre_works_are_ok = False
+                    else:
+                        all_pr_internal_ids = []
+                        for pr in prs:
+                            if pr['status'] not in [
+                                ProcessingStatus.Submitting,
+                                ProcessingStatus.Submitted,
+                                ProcessingStatus.Running,
+                                ProcessingStatus.Finished,
+                                ProcessingStatus.Failed,
+                                ProcessingStatus.FinishedOnStep,
+                                ProcessingStatus.FinishedOnExec,
+                                ProcessingStatus.FinishedTerm,
+                                ProcessingStatus.SubFinished,
+                                ProcessingStatus.Broken,
+                                ProcessingStatus.Terminating,
+                                ProcessingStatus.ToTrigger,
+                                ProcessingStatus.Triggering,
+                                ProcessingStatus.Synchronizing,
+                                ProcessingStatus.Prepared
+                            ]:
+                                pre_works_are_ok = False
+                                break
+                            all_pr_internal_ids.append(pr['internal_id'])
+                        if set(parent_internal_ids) != set(all_pr_internal_ids):
+                            pre_works_are_ok = False
+            else:
+                pre_works_are_ok = True
 
-            if not status:
-                raise exceptions.ProcessSubmitFailed(str(errors))
+            if pre_works_are_ok:
+                ret_new_processing = handle_new_processing(processing,
+                                                           self.agent_attributes,
+                                                           func_site_to_cloud=self.get_site_to_cloud,
+                                                           max_updates_per_round=self.max_updates_per_round,
+                                                           executors=executors,
+                                                           logger=self.logger,
+                                                           log_prefix=log_prefix)
+                status, processing, update_colls, new_contents, new_input_dependency_contents, msgs, errors = ret_new_processing
 
-            parameters = {'status': ProcessingStatus.Prepared,
-                          'substatus': ProcessingStatus.Prepared,
-                          'locking': ProcessingLocking.Idle,
-                          'processing_metadata': processing['processing_metadata']}
-            parameters = self.load_poll_period(processing, parameters, new=True)
+                if not status:
+                    raise exceptions.ProcessSubmitFailed(str(errors))
 
-            update_processing = {'processing_id': processing['processing_id'],
-                                 'parameters': parameters}
-            ret = {'update_processing': update_processing,
-                   'update_collections': update_colls,
-                   'update_contents': [],
-                   'new_contents': new_contents,
-                   'new_input_dependency_contents': new_input_dependency_contents,
-                   'messages': msgs,
-                   }
+                parameters = {'status': ProcessingStatus.Prepared,
+                              'substatus': ProcessingStatus.Prepared,
+                              'locking': ProcessingLocking.Idle,
+                              'processing_metadata': processing['processing_metadata']}
+                parameters = self.load_poll_period(processing, parameters, new=True)
+
+                processing['substatus'] = ProcessingStatus.Prepared
+                update_processing = {'processing_id': processing['processing_id'],
+                                     'parameters': parameters}
+                ret = {'update_processing': update_processing,
+                       'update_collections': update_colls,
+                       'update_contents': [],
+                       'new_contents': new_contents,
+                       'new_input_dependency_contents': new_input_dependency_contents,
+                       'messages': msgs,
+                       }
+            else:
+                parameters = {'locking': ProcessingLocking.Idle}
+                update_processing = {'processing_id': processing['processing_id'],
+                                     'parameters': parameters}
+                ret = {'update_processing': update_processing,
+                       'update_collections': [],
+                       'update_contents': [],
+                       'new_contents': [],
+                       'new_input_dependency_contents': [],
+                       'messages': [],
+                       }
         except Exception as ex:
             self.logger.error(ex)
             self.logger.error(traceback.format_exc())
@@ -298,7 +342,7 @@ class Submitter(Poller):
                    'update_contents': []}
         return ret
 
-    def handle_prepared_processing(self, processing):
+    def handle_prepared_processing(self, processing, check_previous=False):
         try:
             log_prefix = self.get_log_prefix(processing)
 
@@ -309,42 +353,77 @@ class Submitter(Poller):
             if self.enable_executors:
                 executors = self.get_extra_executors()
 
-            ret_new_processing = handle_prepared_processing(processing,
-                                                            self.agent_attributes,
-                                                            func_site_to_cloud=self.get_site_to_cloud,
-                                                            max_updates_per_round=self.max_updates_per_round,
-                                                            executors=executors,
-                                                            logger=self.logger,
-                                                            log_prefix=log_prefix)
-            status, processing, update_colls, new_contents, new_input_dependency_contents, msgs, errors = ret_new_processing
+            if check_previous:
+                pre_works_are_ok = True
+                if processing['parent_internal_id']:
+                    parent_internal_ids = processing['parent_internal_id'].split(",")
+                    prs = core_processings.get_processings(
+                        request_id=processing['request_id'],
+                        internal_ids=parent_internal_ids,
+                        loop_index=processing['loop_index']
+                    )
+                    if not prs:
+                        pre_works_are_ok = False
+                    else:
+                        all_pr_internal_ids = []
+                        for pr in prs:
+                            if not pr['workload_id']:
+                                pre_works_are_ok = False
+                                break
+                            all_pr_internal_ids.append(pr['internal_id'])
+                        if set(parent_internal_ids) != set(all_pr_internal_ids):
+                            pre_works_are_ok = False
+            else:
+                pre_works_are_ok = True
 
-            if not status:
-                raise exceptions.ProcessSubmitFailed(str(errors))
+            if pre_works_are_ok:
+                ret_new_processing = handle_prepared_processing(processing,
+                                                                self.agent_attributes,
+                                                                func_site_to_cloud=self.get_site_to_cloud,
+                                                                max_updates_per_round=self.max_updates_per_round,
+                                                                executors=executors,
+                                                                logger=self.logger,
+                                                                log_prefix=log_prefix)
+                status, processing, update_colls, new_contents, new_input_dependency_contents, msgs, errors = ret_new_processing
 
-            parameters = {'status': ProcessingStatus.Submitting,
-                          'substatus': ProcessingStatus.Submitting,
-                          'locking': ProcessingLocking.Idle,
-                          'processing_metadata': processing['processing_metadata']}
-            parameters = self.load_poll_period(processing, parameters, new=True)
+                if not status:
+                    raise exceptions.ProcessSubmitFailed(str(errors))
 
-            proc = processing['processing_metadata']['processing']
-            if proc.submitted_at:
-                if not processing['submitted_at'] or processing['submitted_at'] < proc.submitted_at:
-                    parameters['submitted_at'] = proc.submitted_at
+                parameters = {'status': ProcessingStatus.Submitting,
+                              'substatus': ProcessingStatus.Submitting,
+                              'locking': ProcessingLocking.Idle,
+                              'processing_metadata': processing['processing_metadata']}
+                parameters = self.load_poll_period(processing, parameters, new=True)
 
-            # if processing['processing_metadata'] and 'processing' in processing['processing_metadata']:
-            if proc.workload_id:
-                parameters['workload_id'] = proc.workload_id
+                proc = processing['processing_metadata']['processing']
+                if proc.submitted_at:
+                    if not processing['submitted_at'] or processing['submitted_at'] < proc.submitted_at:
+                        parameters['submitted_at'] = proc.submitted_at
 
-            update_processing = {'processing_id': processing['processing_id'],
-                                 'parameters': parameters}
-            ret = {'update_processing': update_processing,
-                   'update_collections': update_colls,
-                   'update_contents': [],
-                   'new_contents': new_contents,
-                   'new_input_dependency_contents': new_input_dependency_contents,
-                   'messages': msgs,
-                   }
+                # if processing['processing_metadata'] and 'processing' in processing['processing_metadata']:
+                if proc.workload_id and not processing['workload_id']:
+                    parameters['workload_id'] = proc.workload_id
+
+                update_processing = {'processing_id': processing['processing_id'],
+                                     'parameters': parameters}
+                ret = {'update_processing': update_processing,
+                       'update_collections': update_colls,
+                       'update_contents': [],
+                       'new_contents': new_contents,
+                       'new_input_dependency_contents': new_input_dependency_contents,
+                       'messages': msgs,
+                       }
+            else:
+                parameters = {'locking': ProcessingLocking.Idle}
+                update_processing = {'processing_id': processing['processing_id'],
+                                     'parameters': parameters}
+                ret = {'update_processing': update_processing,
+                       'update_collections': [],
+                       'update_contents': [],
+                       'new_contents': [],
+                       'new_input_dependency_contents': [],
+                       'messages': [],
+                       }
         except Exception as ex:
             self.logger.error(ex)
             self.logger.error(traceback.format_exc())
@@ -356,6 +435,8 @@ class Submitter(Poller):
                 pr_status = ProcessingStatus.Failed
             # increase poll period
             new_poll_period = int(processing['new_poll_period'].total_seconds() * self.poll_period_increase_rate)
+            if new_poll_period < self.new_fail_poll_period:
+                new_poll_period = self.new_fail_poll_period
             if new_poll_period > self.max_new_poll_period:
                 new_poll_period = self.max_new_poll_period
 
@@ -373,10 +454,10 @@ class Submitter(Poller):
                    'update_contents': []}
         return ret
 
-    def process_new_processing(self, event):
+    def process_new_processing(self, event=None, processing=None):
         self.number_workers += 1
         try:
-            if event:
+            if processing is None and event:
                 # pr_status = [ProcessingStatus.New]
                 self.logger.info("process_new_processing, event: %s" % str(event))
                 pr = self.get_processing(processing_id=event._processing_id, status=None, locking=True)
@@ -390,36 +471,51 @@ class Submitter(Poller):
                            'update_contents': []}
                     self.update_processing(ret, pr)
                 else:
-                    log_pre = self.get_log_prefix(pr)
-                    self.logger.info(log_pre + "process_new_processing")
-                    if pr['processing_type'] and pr['processing_type'] in [ProcessingType.iWorkflow, ProcessingType.iWork]:
-                        ret = self.handle_new_iprocessing(pr)
-                        # self.logger.info(log_pre + "process_new_processing result: %s" % str(ret))
+                    processing = pr
+            if processing:
+                pr = processing
+                log_pre = self.get_log_prefix(pr)
+                self.logger.info(log_pre + "process_new_processing")
+                if pr['processing_type'] and pr['processing_type'] in [ProcessingType.iWorkflow, ProcessingType.iWork]:
+                    ret = self.handle_new_iprocessing(pr)
+                    # self.logger.info(log_pre + "process_new_processing result: %s" % str(ret))
 
-                        self.update_processing(ret, pr)
+                    self.update_processing(ret, pr)
 
-                        self.logger.info(log_pre + "UpdateTransformEvent(transform_id: %s)" % pr['transform_id'])
-                        submit_event_content = {'event': 'submitted'}
-                        event = UpdateTransformEvent(publisher_id=self.id, transform_id=pr['transform_id'], content=submit_event_content)
+                    self.logger.info(log_pre + "PreparedProcessingEvent(processing_id: %s)" % pr['processing_id'])
+                    event = PreparedProcessingEvent(publisher_id=self.id, processing_id=pr['processing_id'])
+                    event.set_has_updates()
+                    self.event_bus.send(event)
+                else:
+                    ret = self.handle_new_processing(pr)
+                    self.update_processing(ret, pr)
+                    if processing['substatus'] in [ProcessingStatus.Prepared]:
+                        self.logger.info(log_pre + "PreparedProcessingEvent(processing_id: %s)" % pr['processing_id'])
+                        event = PreparedProcessingEvent(publisher_id=self.id, processing_id=pr['processing_id'])
                         event.set_has_updates()
                         self.event_bus.send(event)
 
-                        self.logger.info(log_pre + "SyncProcessingEvent(processing_id: %s)" % pr['processing_id'])
-                        event = SyncProcessingEvent(publisher_id=self.id, processing_id=pr['processing_id'])
-                        event.set_has_updates()
-                        self.event_bus.send(event)
-                    else:
-                        ret = self.handle_new_processing(pr)
-                        self.update_processing(ret, pr)
+                        # get following processings
+                        following_prs = core_processings.get_processings(
+                            request_id=processing['request_id'],
+                            parent_internal_ids=[processing['internal_id']],
+                            loop_index=processing['loop_index']
+                        )
+                        if following_prs:
+                            for following_pr in following_prs:
+                                self.logger.info(log_pre + "NewProcessingEvent(processing_id: %s)" % following_pr['processing_id'])
+                                event = NewProcessingEvent(publisher_id=self.id, processing_id=following_pr['processing_id'])
+                                self.event_bus.send(event)
+
         except Exception as ex:
             self.logger.error(ex)
             self.logger.error(traceback.format_exc())
         self.number_workers -= 1
 
-    def process_prepared_processing(self, event):
+    def process_prepared_processing(self, event=None, processing=None):
         self.number_workers += 1
         try:
-            if event:
+            if processing is None and event:
                 # pr_status = [ProcessingStatus.New]
                 self.logger.info("process_prepared_processing, event: %s" % str(event))
                 pr = self.get_processing(processing_id=event._processing_id, status=None, locking=True)
@@ -433,33 +529,36 @@ class Submitter(Poller):
                            'update_contents': []}
                     self.update_processing(ret, pr)
                 else:
-                    log_pre = self.get_log_prefix(pr)
-                    self.logger.info(log_pre + "process_prepared_processing")
-                    if pr['processing_type'] and pr['processing_type'] in [ProcessingType.iWorkflow, ProcessingType.iWork]:
-                        # ret = self.handle_new_iprocessing(pr)
-                        self.logger.error(log_pre + "process_prepared_processing, iWorkflow/iWork should not have status prepared")
-                        parameters = {'locking': ProcessingLocking.Idle}
-                        update_processing = {'processing_id': pr['processing_id'],
-                                             'parameters': parameters}
-                        ret = {'update_processing': update_processing,
-                               'update_contents': []}
-                        self.update_processing(ret, pr)
-                    else:
-                        ret = self.handle_prepared_processing(pr)
-                        # self.logger.info(log_pre + "process_new_processing result: %s" % str(ret))
+                    processing = pr
+            if processing:
+                pr = processing
+                log_pre = self.get_log_prefix(pr)
+                self.logger.info(log_pre + "process_prepared_processing")
+                if pr['processing_type'] and pr['processing_type'] in [ProcessingType.iWorkflow, ProcessingType.iWork]:
+                    # ret = self.handle_new_iprocessing(pr)
+                    self.logger.error(log_pre + "process_prepared_processing, iWorkflow/iWork should not have status prepared")
+                    parameters = {'locking': ProcessingLocking.Idle}
+                    update_processing = {'processing_id': pr['processing_id'],
+                                         'parameters': parameters}
+                    ret = {'update_processing': update_processing,
+                           'update_contents': []}
+                    self.update_processing(ret, pr)
+                else:
+                    ret = self.handle_prepared_processing(pr)
+                    # self.logger.info(log_pre + "process_new_processing result: %s" % str(ret))
 
-                        self.update_processing(ret, pr)
+                    self.update_processing(ret, pr)
 
-                        self.logger.info(log_pre + "UpdateTransformEvent(transform_id: %s)" % pr['transform_id'])
-                        submit_event_content = {'event': 'submitted'}
-                        event = UpdateTransformEvent(publisher_id=self.id, transform_id=pr['transform_id'], content=submit_event_content)
-                        event.set_has_updates()
-                        self.event_bus.send(event)
+                    self.logger.info(log_pre + "UpdateTransformEvent(transform_id: %s)" % pr['transform_id'])
+                    submit_event_content = {'event': 'submitted'}
+                    event = UpdateTransformEvent(publisher_id=self.id, transform_id=pr['transform_id'], content=submit_event_content)
+                    event.set_has_updates()
+                    self.event_bus.send(event)
 
-                        self.logger.info(log_pre + "SyncProcessingEvent(processing_id: %s)" % pr['processing_id'])
-                        event = SyncProcessingEvent(publisher_id=self.id, processing_id=pr['processing_id'])
-                        event.set_has_updates()
-                        self.event_bus.send(event)
+                    # self.logger.info(log_pre + "SyncProcessingEvent(processing_id: %s)" % pr['processing_id'])
+                    # event = SyncProcessingEvent(publisher_id=self.id, processing_id=pr['processing_id'])
+                    # event.set_has_updates()
+                    # self.event_bus.send(event)
         except Exception as ex:
             self.logger.error(ex)
             self.logger.error(traceback.format_exc())
@@ -498,6 +597,9 @@ class Submitter(Poller):
             task = self.create_task(task_func=self.get_prepared_processings, task_output_queue=None, task_args=tuple(), task_kwargs={}, delay_time=10, priority=1)
             self.add_task(task)
 
+            task = self.create_task(task_func=self.load_min_request_id, task_output_queue=None, task_args=tuple(), task_kwargs={}, delay_time=600, priority=1)
+            self.add_task(task)
+
             self.execute()
         except KeyboardInterrupt:
             self.stop()
