@@ -6,18 +6,16 @@
 # http://www.apache.org/licenses/LICENSE-2.0OA
 #
 # Authors:
-# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2024
+# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2025
 
 
 """
 operations related to Processings.
 """
 
-import datetime
-
 from idds.orm.base.session import read_session, transactional_session
-from idds.common.constants import ProcessingLocking, ProcessingStatus, ProcessingType, GranularityType, ContentRelationType
-from idds.common.utils import get_list_chunks, get_process_thread_info
+from idds.common.constants import CommandType, ProcessingStatus, ProcessingType, GranularityType, ContentRelationType
+from idds.common.utils import get_list_chunks
 from idds.orm import (processings as orm_processings,
                       collections as orm_collections,
                       contents as orm_contents,
@@ -30,7 +28,9 @@ def add_processing(request_id, workload_id, transform_id, status, submitter=None
                    substatus=ProcessingStatus.New, granularity=None,
                    granularity_type=GranularityType.File,
                    processing_type=ProcessingType.Workflow,
+                   command=CommandType.NoneCommand,
                    new_poll_period=1, update_poll_period=10,
+                   internal_id=None, parent_internal_id=None, loop_index=None,
                    new_retries=0, update_retries=0, max_new_retries=3, max_update_retries=0,
                    expired_at=None, processing_metadata=None, session=None):
     """
@@ -60,6 +60,9 @@ def add_processing(request_id, workload_id, transform_id, status, submitter=None
                                           max_new_retries=max_new_retries,
                                           max_update_retries=max_update_retries,
                                           processing_type=processing_type,
+                                          command=command,
+                                          internal_id=internal_id, parent_internal_id=parent_internal_id,
+                                          loop_index=loop_index,
                                           expired_at=expired_at, processing_metadata=processing_metadata,
                                           session=session)
 
@@ -82,7 +85,8 @@ def get_processing(processing_id=None, request_id=None, transform_id=None, to_js
 
 
 @read_session
-def get_processings(request_id=None, workload_id=None, transform_id=None, to_json=False, session=None):
+def get_processings(request_id=None, workload_id=None, transform_id=None, loop_index=None, internal_ids=None,
+                    parent_internal_ids=None, to_json=False, session=None):
     """
     Get processing or raise a NoObject exception.
 
@@ -94,8 +98,28 @@ def get_processings(request_id=None, workload_id=None, transform_id=None, to_jso
 
     :returns: Processing.
     """
-    return orm_processings.get_processings(request_id=request_id, workload_id=workload_id,
-                                           transform_id=transform_id, to_json=to_json, session=session)
+    prs = orm_processings.get_processings(
+        request_id=request_id,
+        workload_id=workload_id,
+        transform_id=transform_id,
+        loop_index=loop_index,
+        internal_ids=internal_ids,
+        to_json=to_json, session=session
+    )
+    if not prs or not parent_internal_ids:
+        return prs
+
+    if not isinstance(parent_internal_ids, (list, tuple)):
+        parent_internal_ids = parent_internal_ids.split(",")
+
+    ret_prs = []
+    for pr in prs:
+        pr_parent_internal_id = pr['parent_internal_id']
+        if pr_parent_internal_id:
+            pr_parent_internal_id = pr_parent_internal_id.split(",")
+            if any(pid in parent_internal_ids for pid in pr_parent_internal_id):
+                ret_prs.append(pr)
+    return ret_prs
 
 
 @read_session
@@ -115,45 +139,11 @@ def get_processings_by_transform_id(transform_id=None, to_json=False, session=No
 
 
 @transactional_session
-def get_processing_by_id_status(processing_id, status=None, exclude_status=None, locking=False, lock_period=None, session=None):
+def get_processing_by_id_status(processing_id, status=None, exclude_status=None, locking=False, to_lock=False, lock_period=None, session=None):
     # pr = orm_processings.get_processing_by_id_status(processing_id=processing_id, status=status, locking=locking, session=session)
     pr = orm_processings.get_processing_by_id_status(processing_id=processing_id, status=status,
-                                                     exclude_status=exclude_status, session=session)
-    if pr is None:
-        return pr
-
-    if locking:
-        if pr['locking'] in [ProcessingLocking.Locking]:
-            if lock_period and pr['updated_at'] < datetime.datetime.utcnow() - datetime.timedelta(seconds=lock_period):
-                parameters = {}
-                parameters['locking'] = ProcessingLocking.Locking
-                parameters['updated_at'] = datetime.datetime.utcnow()
-                hostname, pid, thread_id, thread_name = get_process_thread_info()
-                parameters['locking_hostname'] = hostname
-                parameters['locking_pid'] = pid
-                parameters['locking_thread_id'] = thread_id
-                parameters['locking_thread_name'] = thread_name
-                num_rows = orm_processings.update_processing(processing_id=pr['processing_id'], parameters=parameters, locking=True, session=session)
-                if num_rows > 0:
-                    return pr
-                else:
-                    return None
-            else:
-                return None
-        else:
-            parameters = {}
-            parameters['locking'] = ProcessingLocking.Locking
-            parameters['updated_at'] = datetime.datetime.utcnow()
-            hostname, pid, thread_id, thread_name = get_process_thread_info()
-            parameters['locking_hostname'] = hostname
-            parameters['locking_pid'] = pid
-            parameters['locking_thread_id'] = thread_id
-            parameters['locking_thread_name'] = thread_name
-            num_rows = orm_processings.update_processing(processing_id=pr['processing_id'], parameters=parameters, locking=True, session=session)
-            if num_rows > 0:
-                return pr
-            else:
-                return None
+                                                     exclude_status=exclude_status, locking=locking,
+                                                     to_lock=to_lock, session=session)
     return pr
 
 
@@ -174,69 +164,14 @@ def get_processings_by_status(status, time_period=None, locking=False, bulk_size
 
     :returns: Processings.
     """
-    if locking:
-        if not only_return_id and bulk_size:
-            # order by cannot work together with locking. So first select 2 * bulk_size without locking with order by.
-            # then select with locking.
-            proc_ids = orm_processings.get_processings_by_status(status=status, period=time_period, locking=locking,
-                                                                 bulk_size=bulk_size * 2, to_json=False, locking_for_update=False,
-                                                                 by_substatus=by_substatus, only_return_id=True,
-                                                                 for_poller=for_poller, new_poll=new_poll,
-                                                                 min_request_id=min_request_id,
-                                                                 update_poll=update_poll, session=session)
-            if proc_ids:
-                processing2s = orm_processings.get_processings_by_status(status=status, period=time_period, locking=locking,
-                                                                         processing_ids=proc_ids,
-                                                                         min_request_id=min_request_id,
-                                                                         bulk_size=None, to_json=to_json,
-                                                                         locking_for_update=locking_for_update,
-                                                                         by_substatus=by_substatus, only_return_id=only_return_id,
-                                                                         new_poll=new_poll, update_poll=update_poll,
-                                                                         for_poller=for_poller, session=session)
-                if processing2s:
-                    # reqs = req2s[:bulk_size]
-                    # order requests
-                    processings = []
-                    for proc_id in proc_ids:
-                        if len(processings) >= bulk_size:
-                            break
-                        for p in processing2s:
-                            if p['processing_id'] == proc_id:
-                                processings.append(p)
-                                break
-                    # processings = processings[:bulk_size]
-                else:
-                    processings = []
-            else:
-                processings = []
-        else:
-            processings = orm_processings.get_processings_by_status(status=status, period=time_period, locking=locking,
-                                                                    bulk_size=bulk_size, to_json=to_json,
-                                                                    locking_for_update=locking_for_update,
-                                                                    new_poll=new_poll, update_poll=update_poll,
-                                                                    only_return_id=only_return_id,
-                                                                    min_request_id=min_request_id,
-                                                                    by_substatus=by_substatus, for_poller=for_poller, session=session)
+    processings = orm_processings.get_processings_by_status(status=status, period=time_period, locking=locking,
+                                                            bulk_size=bulk_size, to_json=to_json,
+                                                            locking_for_update=locking_for_update,
+                                                            new_poll=new_poll, update_poll=update_poll,
+                                                            only_return_id=only_return_id,
+                                                            min_request_id=min_request_id, not_lock=not_lock,
+                                                            by_substatus=by_substatus, for_poller=for_poller, session=session)
 
-        parameters = {}
-        if not not_lock:
-            parameters['locking'] = ProcessingLocking.Locking
-        if next_poll_at:
-            parameters['next_poll_at'] = next_poll_at
-        parameters['updated_at'] = datetime.datetime.utcnow()
-        if parameters:
-            for processing in processings:
-                if type(processing) in [dict]:
-                    orm_processings.update_processing(processing['processing_id'], parameters=parameters, session=session)
-                else:
-                    orm_processings.update_processing(processing, parameters=parameters, session=session)
-    else:
-        processings = orm_processings.get_processings_by_status(status=status, period=time_period, locking=locking,
-                                                                bulk_size=bulk_size, to_json=to_json,
-                                                                new_poll=new_poll, update_poll=update_poll,
-                                                                only_return_id=only_return_id,
-                                                                min_request_id=min_request_id,
-                                                                by_substatus=by_substatus, for_poller=for_poller, session=session)
     return processings
 
 
@@ -256,6 +191,23 @@ def update_processing(processing_id, parameters, session=None):
     return orm_processings.update_processing(processing_id=processing_id, parameters=parameters, session=session)
 
 
+@transactional_session
+def abort_resume_processings(transform_id=None, request_id=None, processing_id=None, abort=False, resume=False, session=None):
+    """
+    abort/resume processings.
+
+    :param request_id: The request id.
+    :param transform_id: The id of the transform.
+    :param session: The database session in use.
+
+    :raises NoObject: If no content is founded.
+    :raises DatabaseException: If there is a database error.
+    """
+    orm_processings.abort_resume_processings(
+        transform_id=transform_id, request_id=request_id, processing_id=processing_id, abort=abort, resume=resume, session=session
+    )
+
+
 @transactional_session
 def delete_processing(processing_id=None, session=None):
     """
@@ -350,7 +302,7 @@ def resolve_input_dependency_id(new_input_dependency_contents, request_id=None,
         # content_dep_id = content_name_id_map[content['coll_id']][content['name']][dep_sub_map_id]
         content_dep_id = content_name_id_map[content['coll_id']][content['name']]
         content['content_dep_id'] = content_dep_id
-        content['name'] = content_dep_id
+        content['name'] = str(content_dep_id)
     return new_input_dependency_contents
 
 
