@@ -205,6 +205,7 @@ class Request(BASE, ModelBase):
     substatus = Column(EnumWithValue(RequestStatus), default=0)
     oldstatus = Column(EnumWithValue(RequestStatus), default=0)
     locking = Column(EnumWithValue(RequestLocking), nullable=False)
+    command = Column(EnumWithValue(CommandType), default=0)
     total_transforms = Column(Integer())
     finished_transforms = Column(Integer())
     subfinished_transforms = Column(Integer())
@@ -371,8 +372,9 @@ class Transform(BASE, ModelBase):
     substatus = Column(EnumWithValue(TransformStatus), default=0)
     oldstatus = Column(EnumWithValue(TransformStatus), default=0)
     locking = Column(EnumWithValue(TransformLocking), nullable=False)
+    command = Column(EnumWithValue(CommandType), default=0)
     retries = Column(Integer(), default=0)
-    parent_internal_id = Column(String(20))
+    parent_internal_id = Column(String(400))
     parent_transform_id = Column(BigInteger())
     previous_transform_id = Column(BigInteger())
     current_processing_id = Column(BigInteger())
@@ -488,11 +490,16 @@ class Processing(BASE, ModelBase):
     status = Column(EnumWithValue(ProcessingStatus), nullable=False)
     substatus = Column(EnumWithValue(ProcessingStatus), default=0)
     oldstatus = Column(EnumWithValue(ProcessingStatus), default=0)
+    loop_index = Column(Integer())
+    internal_id = Column(String(20))
+    parent_internal_id = Column(String(400))
     locking = Column(EnumWithValue(ProcessingLocking), nullable=False)
+    command = Column(EnumWithValue(CommandType), default=0)
     submitter = Column(String(20))
     submitted_id = Column(Integer())
     granularity = Column(Integer())
     granularity_type = Column(EnumWithValue(GranularityType))
+    num_unmapped = Column(Integer(), default=0)
     created_at = Column("created_at", DateTime, default=datetime.datetime.utcnow, nullable=False)
     updated_at = Column("updated_at", DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow, nullable=False)
     next_poll_at = Column("next_poll_at", DateTime, default=datetime.datetime.utcnow)
